@@ -1,9 +1,13 @@
 package Role::Tiny;
+
+sub _getglob { \*{$_[0]} }
+sub _getstash { \%{"$_[0]::"} }
+
 use strict;
 use warnings;
 
-our $VERSION = '2.002_002';
-$VERSION =~ tr/_//d;
+our $VERSION = '2.000001';
+$VERSION = eval $VERSION;
 
 our %INFO;
 our %APPLIED_TO;
@@ -15,20 +19,7 @@ our @ON_ROLE_CREATE;
 
 BEGIN {
   *_WORK_AROUND_BROKEN_MODULE_STATE = "$]" < 5.009 ? sub(){1} : sub(){0};
-  *_WORK_AROUND_HINT_LEAKAGE
-    = "$]" < 5.011 && !("$]" >= 5.009004 && "$]" < 5.010001)
-      ? sub(){1} : sub(){0};
-  *_CONSTANTS_DEFLATE = "$]" >= 5.012 && "$]" < 5.020 ? sub(){1} : sub(){0};
-}
-
-sub _getglob { no strict 'refs'; \*{$_[0]} }
-sub _getstash { no strict 'refs'; \%{"$_[0]::"} }
-
-sub croak {
-  require Carp;
-  no warnings 'redefine';
-  *croak = \&Carp::croak;
-  goto &Carp::croak;
+  *_MRO_MODULE = "$]" < 5.010 ? sub(){"MRO/Compat.pm"} : sub(){"mro.pm"};
 }
 
 sub Role::Tiny::__GUARD__::DESTROY {
@@ -36,132 +27,70 @@ sub Role::Tiny::__GUARD__::DESTROY {
 }
 
 sub _load_module {
-  my ($module) = @_;
-  (my $file = "$module.pm") =~ s{::}{/}g;
-  return 1
-    if $INC{$file};
-
+  (my $proto = $_[0]) =~ s/::/\//g;
+  $proto .= '.pm';
+  return 1 if $INC{$proto};
   # can't just ->can('can') because a sub-package Foo::Bar::Baz
   # creates a 'Baz::' key in Foo::Bar's symbol table
-  return 1
-    if grep !/::\z/, keys %{_getstash($module)};
+  return 1 if grep !/::$/, keys %{_getstash($_[0])||{}};
   my $guard = _WORK_AROUND_BROKEN_MODULE_STATE
-    && bless([ $file ], 'Role::Tiny::__GUARD__');
-  local %^H if _WORK_AROUND_HINT_LEAKAGE;
-  require $file;
+    && bless([ $proto ], 'Role::Tiny::__GUARD__');
+  require $proto;
   pop @$guard if _WORK_AROUND_BROKEN_MODULE_STATE;
   return 1;
 }
 
-sub _require_module {
-  _load_module($_[1]);
-}
-
-sub _all_subs {
-  my ($me, $package) = @_;
-  my $stash = _getstash($package);
-  return {
-    map {;
-      no strict 'refs';
-      # this is an ugly hack to populate the scalar slot of any globs, to
-      # prevent perl from converting constants back into scalar refs in the
-      # stash when they are used (perl 5.12 - 5.18). scalar slots on their own
-      # aren't detectable through pure perl, so this seems like an acceptable
-      # compromise.
-      ${"${package}::${_}"} = ${"${package}::${_}"}
-        if _CONSTANTS_DEFLATE;
-      $_ => \&{"${package}::${_}"}
-    }
-    grep exists &{"${package}::${_}"},
-    grep !/::\z/,
-    keys %$stash
-  };
-}
-
 sub import {
   my $target = caller;
   my $me = shift;
   strict->import;
   warnings->import;
-  my $non_methods = $me->_non_methods($target);
-  $me->_install_subs($target, @_);
-  $me->make_role($target);
-  $me->_mark_new_non_methods($target, $non_methods)
-    if $non_methods && %$non_methods;
-  return;
-}
-
-sub _mark_new_non_methods {
-  my ($me, $target, $old_non_methods) = @_;
-
-  my $non_methods = $INFO{$target}{non_methods};
-
-  my $subs = $me->_all_subs($target);
-  for my $sub (keys %$subs) {
-    if ( exists $old_non_methods->{$sub} && $non_methods->{$sub} != $subs->{$sub} ) {
-      $non_methods->{$sub} = $subs->{$sub};
-    }
-  }
-
-  return;
-}
-
-sub make_role {
-  my ($me, $target) = @_;
-
-  return if $me->is_role($target);
+  return if $me->is_role($target); # already exported into this package
   $INFO{$target}{is_role} = 1;
-
-  my $non_methods = $me->_all_subs($target);
-  delete @{$non_methods}{grep /\A\(/, keys %$non_methods};
-  $INFO{$target}{non_methods} = $non_methods;
-
+  # get symbol table reference
+  my $stash = _getstash($target);
+  # install before/after/around subs
+  foreach my $type (qw(before after around)) {
+    *{_getglob "${target}::${type}"} = sub {
+      require Class::Method::Modifiers;
+      push @{$INFO{$target}{modifiers}||=[]}, [ $type => @_ ];
+      return;
+    };
+  }
+  *{_getglob "${target}::requires"} = sub {
+    push @{$INFO{$target}{requires}||=[]}, @_;
+    return;
+  };
+  *{_getglob "${target}::with"} = sub {
+    $me->apply_roles_to_package($target, @_);
+    return;
+  };
+  # grab all *non-constant* (stash slot is not a scalarref) subs present
+  # in the symbol table and store their refaddrs (no need to forcibly
+  # inflate constant subs into real subs) with a map to the coderefs in
+  # case of copying or re-use
+  my @not_methods = (map { *$_{CODE}||() } grep !ref($_), values %$stash);
+  @{$INFO{$target}{not_methods}={}}{@not_methods} = @not_methods;
   # a role does itself
   $APPLIED_TO{$target} = { $target => undef };
-  foreach my $hook (@ON_ROLE_CREATE) {
-    $hook->($target);
-  }
-}
-
-sub _install_subs {
-  my ($me, $target) = @_;
-  return if $me->is_role($target);
-  my %install = $me->_gen_subs($target);
-  *{_getglob("${target}::${_}")} = $install{$_}
-    for sort keys %install;
-  return;
-}
-
-sub _gen_subs {
-  my ($me, $target) = @_;
-  (
-    (map {;
-      my $type = $_;
-      $type => sub {
-        my $code = pop;
-        my @names = ref $_[0] eq 'ARRAY' ? @{ $_[0] } : @_;
-        push @{$INFO{$target}{modifiers}||=[]}, [ $type, @names, $code ];
-        return;
-      };
-    } qw(before after around)),
-    requires => sub {
-      push @{$INFO{$target}{requires}||=[]}, @_;
-      return;
-    },
-    with => sub {
-      $me->apply_roles_to_package($target, @_);
-      return;
-    },
-  );
+  $_->($target) for @ON_ROLE_CREATE;
 }
 
 sub role_application_steps {
-  qw(
-    _install_methods
-    _check_requires
-    _install_modifiers
-    _copy_applied_list
-  );
+  qw(_install_methods _check_requires _install_modifiers _copy_applied_list);
+}
+
+sub apply_single_role_to_package {
+  my ($me, $to, $role) = @_;
+
+  _load_module($role);
+
+  die "This is apply_role_to_package" if ref($to);
+  die "${role} is not a Role::Tiny" unless $me->is_role($role);
+
+  foreach my $step ($me->role_application_steps) {
+    $me->$step($to, $role);
+  }
 }
 
 sub _copy_applied_list {
@@ -172,6 +101,7 @@ sub _copy_applied_list {
 
 sub apply_roles_to_object {
   my ($me, $object, @roles) = @_;
+  die "No roles supplied!" unless @roles;
   my $class = ref($object);
   # on perl < 5.8.9, magic isn't copied to all ref copies. bless the parameter
   # directly, so at least the variable passed to us will get any magic applied
@@ -182,7 +112,9 @@ my $role_suffix = 'A000';
 sub _composite_name {
   my ($me, $superclass, @roles) = @_;
 
-  my $new_name = $superclass . '__WITH__' . join '__AND__', @roles;
+  my $new_name = join(
+    '__WITH__', $superclass, my $compose_name = join '__AND__', @roles
+  );
 
   if (length($new_name) > 252) {
     $new_name = $COMPOSED{abbrev}{$new_name} ||= do {
@@ -191,124 +123,134 @@ sub _composite_name {
       $abbrev.'__'.$role_suffix++;
     };
   }
-  return $new_name;
+  return wantarray ? ($new_name, $compose_name) : $new_name;
 }
 
 sub create_class_with_roles {
   my ($me, $superclass, @roles) = @_;
 
-  $me->_check_roles(@roles);
+  die "No roles supplied!" unless @roles;
 
-  my $new_name = $me->_composite_name($superclass, @roles);
+  _load_module($superclass);
+  {
+    my %seen;
+    $seen{$_}++ for @roles;
+    if (my @dupes = grep $seen{$_} > 1, @roles) {
+      die "Duplicated roles: ".join(', ', @dupes);
+    }
+  }
 
-  return $new_name
-    if $COMPOSED{class}{$new_name};
+  my ($new_name, $compose_name) = $me->_composite_name($superclass, @roles);
 
-  return $me->_build_class_with_roles($new_name, $superclass, @roles);
-}
+  return $new_name if $COMPOSED{class}{$new_name};
 
-sub _build_class_with_roles {
-  my ($me, $new_name, $superclass, @roles) = @_;
+  foreach my $role (@roles) {
+    _load_module($role);
+    die "${role} is not a Role::Tiny" unless $me->is_role($role);
+  }
+
+  require(_MRO_MODULE);
+
+  my $composite_info = $me->_composite_info_for(@roles);
+  my %conflicts = %{$composite_info->{conflicts}};
+  if (keys %conflicts) {
+    my $fail =
+      join "\n",
+        map {
+          "Method name conflict for '$_' between roles "
+          ."'".join(' and ', sort values %{$conflicts{$_}})."'"
+          .", cannot apply these simultaneously to an object."
+        } keys %conflicts;
+    die $fail;
+  }
+
+  my @composable = map $me->_composable_package_for($_), reverse @roles;
+
+  # some methods may not exist in the role, but get generated by
+  # _composable_package_for (Moose accessors via Moo).  filter out anything
+  # provided by the composable packages, excluding the subs we generated to
+  # make modifiers work.
+  my @requires = grep {
+    my $method = $_;
+    !grep $_->can($method) && !$COMPOSED{role}{$_}{modifiers_only}{$method},
+      @composable
+  } @{$composite_info->{requires}};
+
+  $me->_check_requires(
+    $superclass, $compose_name, \@requires
+  );
+
+  *{_getglob("${new_name}::ISA")} = [ @composable, $superclass ];
+
+  @{$APPLIED_TO{$new_name}||={}}{
+    map keys %{$APPLIED_TO{$_}}, @roles
+  } = ();
 
-  $COMPOSED{base}{$new_name} = $superclass;
-  @{*{_getglob("${new_name}::ISA")}} = ( $superclass );
-  $me->apply_roles_to_package($new_name, @roles);
   $COMPOSED{class}{$new_name} = 1;
   return $new_name;
 }
 
-sub _check_roles {
-  my ($me, @roles) = @_;
-  croak "No roles supplied!" unless @roles;
+# preserved for compat, and apply_roles_to_package calls it to allow an
+# updated Role::Tiny to use a non-updated Moo::Role
 
-  my %seen;
-  if (my @dupes = grep 1 == $seen{$_}++, @roles) {
-    croak "Duplicated roles: ".join(', ', @dupes);
-  }
-
-  foreach my $role (@roles) {
-    $me->_require_module($role);
-    croak "${role} is not a ${me}" unless $me->is_role($role);
-  }
-}
-
-our %BACKCOMPAT_HACK;
-$BACKCOMPAT_HACK{+__PACKAGE__} = 0;
-sub _want_backcompat_hack {
-  my $me = shift;
-  return $BACKCOMPAT_HACK{$me}
-    if exists $BACKCOMPAT_HACK{$me};
-  no warnings 'uninitialized';
-  $BACKCOMPAT_HACK{$me} =
-    $me->can('apply_single_role_to_package') != \&apply_single_role_to_package
-    && $me->can('role_application_steps') == \&role_application_steps
-}
-
-our $IN_APPLY_ROLES;
-sub apply_single_role_to_package {
-  return
-    if $IN_APPLY_ROLES;
-  local $IN_APPLY_ROLES = 1;
-
-  my ($me, $to, $role) = @_;
-  $me->apply_roles_to_package($to, $role);
-}
-
-sub apply_role_to_package {
-  my ($me, $to, $role) = @_;
-  $me->apply_roles_to_package($to, $role);
-}
+sub apply_role_to_package { shift->apply_single_role_to_package(@_) }
 
 sub apply_roles_to_package {
   my ($me, $to, @roles) = @_;
-  croak "Can't apply roles to object with apply_roles_to_package"
-    if ref $to;
 
-  $me->_check_roles(@roles);
+  return $me->apply_role_to_package($to, $roles[0]) if @roles == 1;
 
-  my @have_conflicts;
-  my %role_methods;
+  my %conflicts = %{$me->_composite_info_for(@roles)->{conflicts}};
+  my @have = grep $to->can($_), keys %conflicts;
+  delete @conflicts{@have};
 
-  if (@roles > 1) {
-    my %conflicts = %{$me->_composite_info_for(@roles)->{conflicts}};
-    @have_conflicts = grep $to->can($_), keys %conflicts;
-    delete @conflicts{@have_conflicts};
-
-    if (keys %conflicts) {
-      my $class = $COMPOSED{base}{$to} || $to;
-      my $fail =
-        join "\n",
-          map {
-            "Due to a method name conflict between roles "
-            .join(' and ', map "'$_'", sort values %{$conflicts{$_}})
-            .", the method '$_' must be implemented by '$class'"
-          } sort keys %conflicts;
-      croak $fail;
-    }
-
-    %role_methods = map +($_ => $me->_concrete_methods_of($_)), @roles;
+  if (keys %conflicts) {
+    my $fail =
+      join "\n",
+        map {
+          "Due to a method name conflict between roles "
+          ."'".join(' and ', sort values %{$conflicts{$_}})."'"
+          .", the method '$_' must be implemented by '${to}'"
+        } keys %conflicts;
+    die $fail;
   }
 
-  if (!$IN_APPLY_ROLES and _want_backcompat_hack($me)) {
-    local $IN_APPLY_ROLES = 1;
+  # conflicting methods are supposed to be treated as required by the
+  # composed role. we don't have an actual composed role, but because
+  # we know the target class already provides them, we can instead
+  # pretend that the roles don't do for the duration of application.
+  my @role_methods = map $me->_concrete_methods_of($_), @roles;
+  # separate loops, since local ..., delete ... for ...; creates a scope
+  local @{$_}{@have} for @role_methods;
+  delete @{$_}{@have} for @role_methods;
+
+  # the if guard here is essential since otherwise we accidentally create
+  # a $INFO for something that isn't a Role::Tiny (or Moo::Role) because
+  # autovivification hates us and wants us to die()
+  if ($INFO{$to}) {
+    delete $INFO{$to}{methods}; # reset since we're about to add methods
+  }
+
+  # backcompat: allow subclasses to use apply_single_role_to_package
+  # to apply changes.  set a local var so ours does nothing.
+  our %BACKCOMPAT_HACK;
+  if($me ne __PACKAGE__
+      and exists $BACKCOMPAT_HACK{$me} ? $BACKCOMPAT_HACK{$me} :
+      $BACKCOMPAT_HACK{$me} =
+        $me->can('role_application_steps')
+          == \&role_application_steps
+        && $me->can('apply_single_role_to_package')
+          != \&apply_single_role_to_package
+  ) {
     foreach my $role (@roles) {
       $me->apply_single_role_to_package($to, $role);
     }
   }
-
-  my $role_methods;
-  foreach my $step ($me->role_application_steps) {
-    foreach my $role (@roles) {
-      # conflicting methods are supposed to be treated as required by the
-      # composed role. we don't have an actual composed role, but because
-      # we know the target class already provides them, we can instead
-      # pretend that the roles don't do for the duration of application.
-      $role_methods = $role_methods{$role} and (
-        (local @{$role_methods}{@have_conflicts}),
-        (delete @{$role_methods}{@have_conflicts}),
-      );
-
-      $me->$step($to, $role);
+  else {
+    foreach my $step ($me->role_application_steps) {
+      foreach my $role (@roles) {
+        $me->$step($to, $role);
+      }
     }
   }
   $APPLIED_TO{$to}{join('|',@roles)} = 1;
@@ -317,96 +259,111 @@ sub apply_roles_to_package {
 sub _composite_info_for {
   my ($me, @roles) = @_;
   $COMPOSITE_INFO{join('|', sort @roles)} ||= do {
+    foreach my $role (@roles) {
+      _load_module($role);
+    }
     my %methods;
     foreach my $role (@roles) {
       my $this_methods = $me->_concrete_methods_of($role);
       $methods{$_}{$this_methods->{$_}} = $role for keys %$this_methods;
     }
+    my %requires;
+    @requires{map @{$INFO{$_}{requires}||[]}, @roles} = ();
+    delete $requires{$_} for keys %methods;
     delete $methods{$_} for grep keys(%{$methods{$_}}) == 1, keys %methods;
-    +{ conflicts => \%methods }
+    +{ conflicts => \%methods, requires => [keys %requires] }
   };
 }
 
+sub _composable_package_for {
+  my ($me, $role) = @_;
+  my $composed_name = 'Role::Tiny::_COMPOSABLE::'.$role;
+  return $composed_name if $COMPOSED{role}{$composed_name};
+  $me->_install_methods($composed_name, $role);
+  my $base_name = $composed_name.'::_BASE';
+  # force stash to exist so ->can doesn't complain
+  _getstash($base_name);
+  # Not using _getglob, since setting @ISA via the typeglob breaks
+  # inheritance on 5.10.0 if the stash has previously been accessed an
+  # then a method called on the class (in that order!), which
+  # ->_install_methods (with the help of ->_install_does) ends up doing.
+  { no strict 'refs'; @{"${composed_name}::ISA"} = ( $base_name ); }
+  my $modifiers = $INFO{$role}{modifiers}||[];
+  my @mod_base;
+  my @modifiers = grep !$composed_name->can($_),
+    do { my %h; @h{map @{$_}[1..$#$_-1], @$modifiers} = (); keys %h };
+  foreach my $modified (@modifiers) {
+    push @mod_base, "sub ${modified} { shift->next::method(\@_) }";
+  }
+  my $e;
+  {
+    local $@;
+    eval(my $code = join "\n", "package ${base_name};", @mod_base);
+    $e = "Evaling failed: $@\nTrying to eval:\n${code}" if $@;
+  }
+  die $e if $e;
+  $me->_install_modifiers($composed_name, $role);
+  $COMPOSED{role}{$composed_name} = {
+    modifiers_only => { map { $_ => 1 } @modifiers },
+  };
+  return $composed_name;
+}
+
 sub _check_requires {
   my ($me, $to, $name, $requires) = @_;
-  $requires ||= $INFO{$name}{requires} || [];
-  if (my @requires_fail = grep !$to->can($_), @$requires) {
+  return unless my @requires = @{$requires||$INFO{$name}{requires}||[]};
+  if (my @requires_fail = grep !$to->can($_), @requires) {
     # role -> role, add to requires, role -> class, error out
     if (my $to_info = $INFO{$to}) {
       push @{$to_info->{requires}||=[]}, @requires_fail;
     } else {
-      croak "Can't apply ${name} to ${to} - missing ".join(', ', @requires_fail);
+      die "Can't apply ${name} to ${to} - missing ".join(', ', @requires_fail);
     }
   }
 }
 
-sub _non_methods {
-  my ($me, $role) = @_;
-  my $info = $INFO{$role} or return {};
-
-  my %non_methods = %{ $info->{non_methods} || {} };
-
-  # this is only for backwards compatibility with older Moo, which
-  # reimplements method tracking rather than calling our method
-  my %not_methods = reverse %{ $info->{not_methods} || {} };
-  return \%non_methods unless keys %not_methods;
-
-  my $subs = $me->_all_subs($role);
-  for my $sub (grep !/\A\(/, keys %$subs) {
-    my $code = $subs->{$sub};
-    if (exists $not_methods{$code}) {
-      $non_methods{$sub} = $code;
-    }
-  }
-
-  return \%non_methods;
-}
-
 sub _concrete_methods_of {
   my ($me, $role) = @_;
   my $info = $INFO{$role};
-
-  return $info->{methods}
-    if $info && $info->{methods};
-
-  my $non_methods = $me->_non_methods($role);
-
-  my $subs = $me->_all_subs($role);
-  for my $sub (keys %$subs) {
-    if ( exists $non_methods->{$sub} && $non_methods->{$sub} == $subs->{$sub} ) {
-      delete $subs->{$sub};
-    }
-  }
-
-  if ($info) {
-    $info->{methods} = $subs;
-  }
-  return $subs;
+  # grab role symbol table
+  my $stash = _getstash($role);
+  # reverse so our keys become the values (captured coderefs) in case
+  # they got copied or re-used since
+  my $not_methods = { reverse %{$info->{not_methods}||{}} };
+  $info->{methods} ||= +{
+    # grab all code entries that aren't in the not_methods list
+    map {
+      my $code = *{$stash->{$_}}{CODE};
+      ( ! $code or exists $not_methods->{$code} ) ? () : ($_ => $code)
+    } grep !ref($stash->{$_}), keys %$stash
+  };
 }
 
 sub methods_provided_by {
   my ($me, $role) = @_;
-  $me->_require_module($role);
-  croak "${role} is not a ${me}" unless $me->is_role($role);
-  sort (keys %{$me->_concrete_methods_of($role)}, @{$INFO{$role}->{requires}||[]});
+  die "${role} is not a Role::Tiny" unless $me->is_role($role);
+  (keys %{$me->_concrete_methods_of($role)}, @{$INFO{$role}->{requires}||[]});
 }
 
 sub _install_methods {
   my ($me, $to, $role) = @_;
 
+  my $info = $INFO{$role};
+
   my $methods = $me->_concrete_methods_of($role);
 
-  my %existing_methods;
-  @existing_methods{keys %{ $me->_all_subs($to) }} = ();
+  # grab target symbol table
+  my $stash = _getstash($to);
 
-  # _concrete_methods_of caches its result on roles.  that cache needs to be
-  # invalidated after applying roles
-  delete $INFO{$to}{methods} if $INFO{$to};
-
-  foreach my $i (keys %$methods) {
-    next
-      if exists $existing_methods{$i};
+  # determine already extant methods of target
+  my %has_methods;
+  @has_methods{grep
+    +(ref($stash->{$_}) || *{$stash->{$_}}{CODE}),
+    keys %$stash
+  } = ();
 
+  foreach my $i (grep !exists $has_methods{$_}, keys %$methods) {
+    no warnings 'once';
     my $glob = _getglob "${to}::${i}";
     *$glob = $methods->{$i};
 
@@ -417,7 +374,7 @@ sub _install_methods {
         && ((defined &overload::nil && $methods->{$i} == \&overload::nil)
             || (defined &overload::_nil && $methods->{$i} == \&overload::_nil));
 
-    my $overload = ${ _getglob "${role}::${i}" };
+    my $overload = ${ *{_getglob "${role}::${i}"}{SCALAR} };
     next
       unless defined $overload;
 
@@ -430,16 +387,10 @@ sub _install_methods {
 sub _install_modifiers {
   my ($me, $to, $name) = @_;
   return unless my $modifiers = $INFO{$name}{modifiers};
-  my $info = $INFO{$to};
-  my $existing = ($info ? $info->{modifiers} : $COMPOSED{modifiers}{$to}) ||= [];
-  my @modifiers = grep {
-    my $modifier = $_;
-    !grep $_ == $modifier, @$existing;
-  } @{$modifiers||[]};
-  push @$existing, @modifiers;
-
-  if (!$info) {
-    foreach my $modifier (@modifiers) {
+  if (my $info = $INFO{$to}) {
+    push @{$info->{modifiers}}, @{$modifiers||[]};
+  } else {
+    foreach my $modifier (@{$modifiers||[]}) {
       $me->_install_single_modifier($to, @$modifier);
     }
   }
@@ -451,11 +402,9 @@ sub _install_single_modifier {
   my ($me, @args) = @_;
   defined($vcheck_error) or $vcheck_error = do {
     local $@;
-    eval {
-      require Class::Method::Modifiers;
-      Class::Method::Modifiers->VERSION(1.05);
-      1;
-    } ? 0 : $@;
+    eval { Class::Method::Modifiers->VERSION(1.05); 1 }
+      ? 0
+      : $@
   };
   $vcheck_error and die $vcheck_error;
   Class::Method::Modifiers::install_modifier(@args);
@@ -484,35 +433,10 @@ sub _install_does {
   return *{_getglob "${to}::DOES"} = $new_sub;
 }
 
-sub _linear_isa($;$) {
-  if (defined &mro::get_linear_isa) {
-    no warnings 'redefine', 'prototype';
-    *_linear_isa = \&mro::get_linear_isa;
-    goto &mro::get_linear_isa;
-  }
-
-  my @check = shift;
-  my @lin;
-
-  my %found;
-  while (defined(my $check = shift @check)) {
-      push @lin, $check;
-      no strict 'refs';
-      unshift @check, grep !$found{$_}++, @{"$check\::ISA"};
-  }
-
-  return \@lin;
-}
-
-# use core function when possible
-if (defined &mro::get_linear_isa) {
-  no warnings 'redefine', 'prototype';
-  *_linear_isa = \&mro::get_linear_isa;
-}
-
 sub does_role {
   my ($proto, $role) = @_;
-  foreach my $class (@{_linear_isa(ref($proto)||$proto)}) {
+  require(_MRO_MODULE);
+  foreach my $class (@{mro::get_linear_isa(ref($proto)||$proto)}) {
     return 1 if exists $APPLIED_TO{$class}{$role};
   }
   return 0;
@@ -520,14 +444,7 @@ sub does_role {
 
 sub is_role {
   my ($me, $role) = @_;
-  return !!($INFO{$role} && (
-    $INFO{$role}{is_role}
-    # these are for backward compatibility with older Moo that overrode some
-    # methods without calling the originals, thus not getting is_role set
-    || $INFO{$role}{requires}
-    || $INFO{$role}{not_methods}
-    || $INFO{$role}{non_methods}
-  ));
+  return !!($INFO{$role} && $INFO{$role}{is_role});
 }
 
 1;
@@ -537,7 +454,7 @@ __END__
 
 =head1 NAME
 
-Role::Tiny - Roles: a nouvelle cuisine portion size slice of Moose
+Role::Tiny - Roles. Like a nouvelle cuisine portion size slice of Moose.
 
 =head1 SYNOPSIS
 
@@ -549,11 +466,11 @@ Role::Tiny - Roles: a nouvelle cuisine portion size slice of Moose
 
  sub bar { ... }
 
- around baz => sub { ... };
+ around baz => sub { ... }
 
  1;
 
-elsewhere
+else where
 
  package Some::Class;
 
@@ -585,8 +502,7 @@ inheritance.  The basics of this implementation of roles is:
 =item *
 
 If a method is already defined on a class, that method will not be composed in
-from the role. A method inherited by a class gets overridden by the role's
-method of the same name, though.
+from the role.
 
 =item *
 
@@ -601,22 +517,6 @@ are applied in a single call (single with statement), then if any of their
 provided methods clash, an exception is raised unless the class provides
 a method since this conflict indicates a potential problem.
 
-=head2 ROLE METHODS
-
-All subs created after importing Role::Tiny will be considered methods to be
-composed. For example:
-
-    package MyRole;
-    use List::Util qw(min);
-    sub mysub { }
-    use Role::Tiny;
-    use List::Util qw(max);
-    sub mymethod { }
-
-In this role, C<max> and C<mymethod> will be included when composing MyRole,
-and C<min> and C<mysub> will not. For additional control, L<namespace::clean>
-can be used to exclude undesired subs from roles.
-
 =head1 IMPORTED SUBROUTINES
 
 =head2 requires
@@ -646,7 +546,7 @@ different roles, please refactor your codebase.
 
  before foo => sub { ... };
 
-See L<< Class::Method::Modifiers/before method(s) => sub { ... }; >> for full
+See L<< Class::Method::Modifiers/before method(s) => sub { ... } >> for full
 documentation.
 
 Note that since you are not required to use method modifiers,
@@ -658,7 +558,7 @@ both L<Class::Method::Modifiers> and L<Role::Tiny>.
 
  around foo => sub { ... };
 
-See L<< Class::Method::Modifiers/around method(s) => sub { ... }; >> for full
+See L<< Class::Method::Modifiers/around method(s) => sub { ... } >> for full
 documentation.
 
 Note that since you are not required to use method modifiers,
@@ -670,7 +570,7 @@ both L<Class::Method::Modifiers> and L<Role::Tiny>.
 
  after foo => sub { ... };
 
-See L<< Class::Method::Modifiers/after method(s) => sub { ... }; >> for full
+See L<< Class::Method::Modifiers/after method(s) => sub { ... } >> for full
 documentation.
 
 Note that since you are not required to use method modifiers,
@@ -708,12 +608,6 @@ hierarchy provides C<DOES>, then Role::Tiny will not override it.
 
 =head1 METHODS
 
-=head2 make_role
-
- Role::Tiny->make_role('Some::Role');
-
-Makes a package into a role, but does not export any subs into it.
-
 =head2 apply_roles_to_package
 
  Role::Tiny->apply_roles_to_package(
@@ -726,9 +620,8 @@ Composes role with package.  See also L<Role::Tiny::With>.
 
  Role::Tiny->apply_roles_to_object($foo, qw(Some::Role1 Some::Role2));
 
-Composes roles in order into object directly. Object is reblessed into the
-resulting class. Note that the object's methods get overridden by the role's
-ones with the same names.
+Composes roles in order into object directly.  Object is reblessed into the
+resulting class.
 
 =head2 create_class_with_roles
 
@@ -748,10 +641,7 @@ Returns true if the given package is a role.
 =over 4
 
 =item * On perl 5.8.8 and earlier, applying a role to an object won't apply any
-overloads from the role to other copies of the object.
-
-=item * On perl 5.16 and earlier, applying a role to a class won't apply any
-overloads from the role to any existing instances of the class.
+overloads from the role to all copies of the object.
 
 =back
 
